<?php

/**
 * This file is part of ILIAS, a powerful learning management system
 * published by ILIAS open source e-Learning e.V.
 *
 * ILIAS is licensed with the GPL-3.0,
 * see https://www.gnu.org/licenses/gpl-3.0.en.html
 * You should have received a copy of said license along with the
 * source code, too.
 *
 * If this is not the case or you just want to try ILIAS, you'll find
 * us at:
 * https://www.ilias.de
 * https://github.com/ILIAS-eLearning
 *
 *********************************************************************/

declare(strict_types=1);

namespace ILIAS\Plugin\PlagScanCron;

use ilCheckboxInputGUI;
use ilCronJob;
use ilCronJobResult;
use ilCronManager;
use ilCronManagerImpl;
use ilDBInterface;
use ILIAS\Plugin\PlagScanExercise\PlagScan\DocumentManagement;
use ilPlagScanCronPlugin;
use ilPropertyFormGUI;
use ilSetting;
use ilUtil;

class Task extends ilCronJob
{
    protected ilPlagScanCronPlugin $plugin;
    protected ilSetting $settings;
    private ilCronManager $cron_manager;
    protected bool $hotrun = false;
    protected bool $protocol = false;

    public function __construct()
    {
        global $DIC;

        $this->plugin = ilPlagScanCronPlugin::getInstance();
        $this->settings = new ilSetting('psxcron');
        $this->cron_manager = $DIC->cron()->manager();
        $this->hotrun = (bool) $this->settings->get('hotrun', '');
        $this->protocol = (bool) $this->settings->get('protocol', '');
    }

    public function getId(): string
    {
        return 'psxcron_job';
    }

    public function hasAutoActivation(): bool
    {
        return true;
    }

    public function hasFlexibleSchedule(): bool
    {
        return true;
    }

    public function getDefaultScheduleType(): int
    {
        return self::SCHEDULE_TYPE_IN_MINUTES;
    }

    public function getDefaultScheduleValue(): int
    {
        return 5;
    }

    public function hasCustomSettings(): bool
    {
        return true;
    }

    public function addCustomSettingsToForm(ilPropertyFormGUI $form): void
    {
        $activate = new ilCheckboxInputGUI($this->plugin->txt('sysconf_activate'), 'activation');
        $activate->setInfo($this->plugin->txt('sysconf_activate_info'));
        $activate->setChecked((bool) $this->settings->get('activation'));
        $form->addItem($activate);

        $hotrun = new ilCheckboxInputGUI($this->plugin->txt('hotrun'), 'hotrun');
        $hotrun->setInfo($this->plugin->txt('hotrun_info'));
        $hotrun->setChecked((bool) $this->settings->get('hotrun'));
        $form->addItem($hotrun);

        $protocol = new ilCheckboxInputGUI($this->plugin->txt('protocol'), 'protocol');
        $protocol->setInfo($this->plugin->txt('protocol_info'));
        $protocol->setChecked((bool) $this->settings->get('protocol'));
        $form->addItem($protocol);
    }

    public function saveCustomSettings(ilPropertyFormGUI $form): bool
    {
        $this->settings->set('activation', $form->getInput('activation'));
        $this->settings->set('hotrun', $form->getInput('hotrun'));
        $this->settings->set('protocol', $form->getInput('protocol'));

        return true;
    }

    public function activationWasToggled(ilDBInterface $db, ilSetting $setting, bool $currently_active): void
    {
        $this->settings->set('activation', $currently_active ? '1' : '0');
        parent::activationWasToggled($db, $setting, $currently_active);
    }

    public function getTitle(): string
    {
        return $this->plugin->txt('cronjob_title');
    }

    public function getDescription(): string
    {
        $txt = $this->plugin->txt('cronjob_description');
        $txt .= '<br /><em>' . $this->plugin->txt('cronjob_status') . '</em>';

        if ($this->settings->get('hotrun', '0') === '1') {
            $txt .= ' ' . $this->plugin->txt('hotrun_active');
        } else {
            $txt .= ' ' . $this->plugin->txt('dryrun_active');
        }

        return $txt;
    }

    public function run(): ilCronJobResult
    {
        $this->cron_manager->ping($this->getId());

        if ($this->hotrun) {
            DocumentManagement::checkAllDocuments();
            $this->cron_manager->ping($this->getId());

            DocumentManagement::updateAllDocuments();
            $this->cron_manager->ping($this->getId());
        }
        $this->cron_manager->ping($this->getId());

        $result = new ilCronJobResult();
        $result->setStatus(ilCronJobResult::STATUS_OK);

        return $result;
    }
}
